subroutine energy_evolution(t,total_number,v,particle_weight,efield,ngrids)
  use precision,only: p_
  use  spatial_grids,only: dx,width
  implicit none
  integer,intent(in):: total_number,ngrids
  real(p_),intent(in):: t,v(total_number),particle_weight(total_number),efield(0:ngrids)
  real(p_):: sum1,sum2
  integer:: i

  sum1=0._p_
  do i=1,total_number
     sum1=sum1+v(i)**2*particle_weight(i)
  enddo
  sum1=sum1 !particle energy

  sum2=0._p_
  do i=0,ngrids-1
     sum2=sum2+(0.5_p_*(efield(i)+efield(i+1)))**2*dx
  enddo
!  sum2=sum2/width  !field energy
  sum2=sum2  !field energy

  write(101,*) t,sum1, sum2
end subroutine energy_evolution


subroutine fourier_evolution(t,efield,ngrids)
  use precision,only: p_
  use constants,only: twopi
  use  spatial_grids,only: dx,width
  implicit none
  integer,intent(in):: ngrids
  real(p_),intent(in)::t, efield(0:ngrids-1)
  complex(p_):: efield_hat(0:ngrids-1),ii=(0._p_,1._p_)
  integer,parameter:: n_harmonic=20
  real(p_):: cos_part(n_harmonic),sin_part(n_harmonic),kvector(n_harmonic)
  integer:: j

  call my_fft(ngrids,dx,efield,efield_hat) !check the spatial Fourier components of efield

  efield_hat=efield_hat/ngrids !to get the Fourier expansion coefficients

  do j=1,n_harmonic
     kvector(j)=j*twopi/width !the corresponding wavenumber
     cos_part(j)=efield_hat(j)+efield_hat(ngrids-j)
     sin_part(j)=-ii*(efield_hat(j)-efield_hat(ngrids-j))
  enddo

  write(102,*) t,(kvector(j),cos_part(j),sin_part(j),j=1,n_harmonic) !

end subroutine fourier_evolution

subroutine reduced_distribution(total_number,r,v,particle_weight,file1,file2)
  use  precision,only:p_
  use  constants,only:zero,one,four,pi
  use spatial_grids,only:width
  use velocity_space,only: vmin,vmax
  implicit none
  integer,intent(in)::total_number
  real(p_),intent(in):: r(total_number),v(total_number),particle_weight(total_number)
  character(*),intent(in):: file1,file2
  !integer,intent(in):: sample_number
  integer,parameter::sample_number=100
  real(p_):: starting_value,ending_value,sum
  real(p_):: xcenter(sample_number-1),possibility_density(sample_number-1)
  integer:: i

  starting_value=zero
  ending_value=width
  call calculate_possibility_density(r, particle_weight,total_number,sample_number,&
       & starting_value,ending_value,xcenter,possibility_density)
  open(33,file=file1)
  do i=1,sample_number-1
     write(33,*) xcenter(i),possibility_density(i)
  enddo
  close(33) 

  starting_value=vmin
  ending_value=vmax
  call calculate_possibility_density(v,particle_weight, total_number,sample_number,&
       & starting_value,ending_value,xcenter,possibility_density)
  open(33,file=file2)
  do i=1,sample_number-1
     write(33,*) xcenter(i), possibility_density(i),possibility_density(i)/(exp(-xcenter(i)**2)/sqrt(pi))
  enddo
  close(33)

  sum=0._p_
  do i=1,sample_number-1
     sum=sum+possibility_density(i)*(ending_value-starting_value)/(sample_number-1)
  enddo
  !  write(*,*) 'total_possibility=', sum !the result should be 1.0

!!$  sum=0._p_
!!$  do i=1,sample_number-1
!!$     sum=sum+0.5_p_*xcenter(i)**2*possibility_density(i)*(ending_value-starting_value)/(sample_number-1)
!!$  enddo
!!$  write(*,*) 'kinetic energy=', sum 

end subroutine reduced_distribution

subroutine calculate_possibility_density(v,particle_weight,total_number,sample_number,&
     & starting_value,ending_value,xcenter,possibility_density)
    use  precision,only:p_
    use  constants,only:one,two
    implicit none
    integer,intent(in):: total_number,sample_number
    real(p_),intent(in):: v(total_number),particle_weight(total_number)
    real(p_),intent(in):: starting_value,ending_value
    real(p_),intent(out):: xcenter(sample_number-1)
    real(p_),intent(out):: possibility_density(sample_number-1)
    real(p_):: bin_npt(sample_number-1)
    real(p_):: interval
    real(p_):: x(sample_number)
    integer:: i,j,k

    interval=(ending_value-starting_value)/(sample_number-1)
    do i=1,sample_number
       x(i)=starting_value+interval*(i-1)
    enddo

    bin_npt=0
!!$  do i=1,sample_number-1
!!$     do j=1,total_number
!!$        if (v(j).ge.x(i) .and. v(j).le.x(i+1)) bin_npt(i)=bin_npt(i)+1
!!$     enddo
!!$  enddo
    do j=1,total_number
       call location2(sample_number,x,v(j),k)
       !bin_npt(k)=bin_npt(k)+1
       bin_npt(k)=bin_npt(k)+particle_weight(j)
       !        if (v(j).ge.x(i) .and. v(j).le.x(i+1)) bin_npt(i)=bin_npt(i)+1
    enddo

    do i=1,sample_number-1
       xcenter(i)=(x(i)+x(i+1))/two
       !possibility_density(i)=bin_npt(i)/(total_number)/interval
       !possibility_density(i)=bin_npt(i)/(total_number)*sample_number
       possibility_density(i)=bin_npt(i)/(total_number)/interval
    enddo

end subroutine calculate_possibility_density


  subroutine location2(n,x,xval,k) 
    !use bisection method to locate xval in an array
    !return k (xval is located between x(k) and x(k+1)
    use precision,only:p_
    implicit none
    integer,intent(in):: n
    real(p_),intent(in):: x(1:n),xval
    integer,intent(out)::k
    integer:: kl,ku,km

    kl=1
    ku=n
30  if(ku-kl .gt. 1) then
       km=(ku+kl)/2
       if((x(n).ge.x(1)).eqv.(xval.ge.x(km))) then
          kl=km
       else
          ku=km
       endif
       goto 30
    endif
    k=kl
  end subroutine location2


subroutine record_field(ngrids,x,rho,phi,efield,filename)
  use precision,only:p_
  implicit none
  integer,intent(in):: ngrids
  real(p_),intent(in):: x(0:ngrids),rho(0:ngrids),phi(0:ngrids),efield(0:ngrids)
  character(*),intent(in):: filename
  integer:: i

  open(33,file=filename)
  do i=0,ngrids
     write(33,*) x(i),rho(i),phi(i),efield(i)
  enddo
  close(33)

end subroutine record_field


subroutine  phase_space_structure(r,v,deltaf,total_number,filename)
  use  precision,only:p_
  use constants,only:pi
  implicit none
  integer,intent(in)::total_number
  real(p_),intent(in):: r(total_number),v(total_number),deltaf(total_number)
  character(*),intent(in):: filename
  integer:: i
  open(33,file=filename)
  do i=1, total_number
!     write(33,*) r(i),v(i) !to plot phase plot
     write(33,*) r(i),v(i),deltaf(i),deltaf(i)/(exp(-v(i)**2)/sqrt(pi)) !the structure of deltaf/f0 in phase space
  enddo
  close(33)
end subroutine phase_space_structure



