subroutine density(total_number,particle_weight,r,x,rho,ngrids) !use PIC (Particle In Cell) method to calculate the perturbed charge density at grids
  use  precision,only:p_
  use  constants,only:zero,one,two,twopi
  use  spatial_grids,only: dx
  implicit none
  integer,intent(in):: total_number
  real(p_),intent(in):: particle_weight(total_number),r(total_number)
  integer,intent(in):: ngrids
  real(p_),intent(in):: x(0:ngrids)
  real(p_),intent(out):: rho(0:ngrids)
  integer:: nloc,i
  real(p_):: w1,w2 !,factor

  rho=0._p_
  do i=1,total_number
     call location(ngrids,x,r(i),nloc)
     w1=(x(nloc+1)-r(i))/(x(nloc+1)-x(nloc))
     w2=one-w1
     rho(nloc)=rho(nloc)+w1*particle_weight(i)
     rho(nloc+1)=rho(nloc+1)+w2*particle_weight(i)
  enddo
!for periodic case, the above treatments miss the contribution to the nloc=0 grid from the left cell, and also miss the contribution to the nloc=ngrids-1 grid from the right cell, summing the rho(0) and rho(ngrids) given above gives the desired result
  rho(0)=rho(0)+rho(ngrids) !rho(ngrids) contains the contribution from the left half-cell of nloc=0
  rho(ngrids)=rho(0) !periodic condition

!  rho=rho*(ngrids+1) !normalized to 1 on every grid, missing the (ngrids+1) factor is one of the important bugs I found.
  rho=rho/dx
  rho=-rho 
end subroutine density


subroutine location(n,x,xval,k) 
!use bisection method to locate xval in an array
 !return k (xval is located between x(k) and x(k+1)
  use precision,only:p_
  implicit none
  integer,intent(in):: n
  real(p_),intent(in):: x(0:n),xval
  integer,intent(out)::k
  integer:: kl,ku,km

  kl=0
  ku=n
30 if(ku-kl .gt. 1) then
     km=(ku+kl)/2
     if((x(n).ge.x(0)).eqv.(xval.ge.x(km))) then
        kl=km
     else
        ku=km
     endif
     goto 30
  endif
  k=kl
end subroutine location


