!!$CALL init_random_seed()         ! see example of RANDOM_SEED
!!$            CALL RANDOM_NUMBER(r)

subroutine loading_particles(loading_scheme,total_number,r,v,phase_space_vol)
  use  precision,only:p_
  use  constants,only:zero,one,two,pi,twopi
  use spatial_grids,only: xmin,xmax,width
  use velocity_space,only: vmin,vmax
  implicit none

  integer,intent(in):: loading_scheme,total_number
  real(p_),intent(out):: r(total_number),v(total_number),phase_space_vol(total_number)
  real(p_):: p(total_number) !posibility function used in making phase space sampling

  real(p_):: random_yj !function name
  real(p_)::sdf,vdf !function name
  external:: sdf,vdf
  real(p_):: sdf_max,vdf_max

  integer:: i
  if (loading_scheme.eq.1) then !load particles accroding to the equilibrium distribution function
       do i=1,total_number 
        !call spatial_distribution(xmin,xmax,r(i))
        !call velocity_distribution(vmin,vmax,v(i))
        sdf_max=one/(xmax-xmin) !maximum value of the distribution function
        call generate_distribution(sdf,sdf_max,xmin,xmax,r(i))
        vdf_max=one/sqrt(pi) !maximum value of the distribution function
        call generate_distribution(vdf,vdf_max,vmin,vmax,v(i))
     enddo
   do i=1,total_number
        p(i)=sdf(xmin,xmax,r(i))*vdf(vmin,vmax,v(i)) !posibility function used in making phase space sampling
     enddo
  else if (loading_scheme.eq.2) then !load particles uniformly
     p=1._p_/((xmax-xmin)*(vmax-vmin)) !probability density for the random sample in the phase space
     do i=1,total_number 
        r(i)=random_yj(0)*width
        v(i)=(random_yj(0) -0.5_p_)*two*vmax
     enddo
!!$  CALL init_random_seed()         ! see example of RANDOM_SEED
!!$  CALL RANDOM_NUMBER(r)
!!$  CALL RANDOM_NUMBER(v)
!!$  r=r*width
!!$  v=(v-0.5_p_)*two*vmax
  else 
     stop 'please specify a loading scheme'
  endif
  phase_space_vol=one/(total_number*p) !phase_space_vol(j) is the phase space volume occupied by the jth particle marker.
end subroutine loading_particles


subroutine generate_distribution(py,pymax,ymin,ymax,z)
  !use rejection method to generate one value which satisfies stitistically the given possiblility distrubiton function
  use precision,only:p_
  use constants,only:twopi,one,two,four
  implicit none
  real(p_),external:: py !distribution function
  real(p_),intent(in):: ymin,ymax,pymax
  real(p_),intent(out):: z
  real(p_):: possibility
  real(p_):: random_yj !functio name

  integer,parameter:: max_try=10000
  integer:: i
  real(p_):: xj2,yj2,tmp

  do i=1,max_try
     yj2=ymin+(ymax-ymin)*random_yj(0)  ! a random value in the range [ymin:ymax]
     possibility=py(ymin,ymax,yj2)
     xj2=pymax*random_yj(0)   !a random value in the range [0: pymax]
     if(possibility<xj2) then
        cycle
     else
        z=yj2
        return
     endif
  enddo
  write(*,*) 'warning**, not successful in generating'
end subroutine generate_distribution


function sdf(ymin,ymax,y) !spatial distribution function
  use precision,only:p_
  use constants,only:one
  implicit none
  real(p_):: sdf,ymin,ymax,y
  sdf=one/(ymax-ymin) !to enure the total probability to be unity.
end function sdf

function vdf(vmin,vmax,v) !velocity distribution function
  use precision,only:p_
  use constants,only:two
  use velocity_space,only:vb1,vb2,vt
  implicit none
  real(p_):: vdf,vmin,vmax,v !vmin and vmax are actually not used. They are included to make function vdf and sdf accept a similar form of arguments.
  real(p_):: gauss

  vdf=(gauss(vb1,vt,v)+gauss(vb2,vt,v))/two
end function vdf

function gauss(vb,vt,v)
  use precision,only:p_
  use constants,only:pi
  implicit none
  real(p_):: gauss,vb,vt,v
  !gauss=exp(-(v-vb)**2/(2*vt**2))/(sqrt(twopi)*vt)
  gauss=exp(-(v-vb)**2/(vt**2))/(sqrt(pi)*vt)
end function gauss

function random_yj(seed) result (z) !return a random number uniform distribute in [0:1]
  !linear congruental method to genereate random number
  !This corresponds to Park and Miller's choice implemented using Schrange's algorithm to aviod integer overflow
  !refer to R. Fitzpatrick's book "Computational physics, an introduction course" for the details
  use precision,only:p_
  implicit none
  integer:: seed
  real(p_):: z 
!!$  integer, parameter:: a=106,c=1283,m=6075
!!$    z=mod(a*seed+c,m)
  integer, parameter:: a=16807,m=2147483647 !m=2^31-1,c=0, this choice is called Park and Miller method
  integer,parameter:: q=127773 !q=m/a
  integer,parameter:: r=2836 !r=mod(m,a)
  real(p_),parameter:: RANDMAX=2147483646._p_
  integer,save:: next=1
  !  write(*,*) m
  if (seed .ne.0) next=seed
  next=a*mod(next,q)-r*(next/q)
  if(next<0) next=next+m
  z=next/RANDMAX
end function random_yj




