subroutine efield_solver_dft(ngrids,dx,rho,efield) 
  !use the FFT method to solve Poisson's equation to get the electric potential phi
  !(use FFTW library)
  use, intrinsic :: iso_c_binding
  use  precision,only:p_
  use  constants,only:zero,one,twopi
  implicit none
  include 'fftw3.f03'
  integer,intent(in):: ngrids
  real(p_),intent(in):: dx,rho(0:ngrids-1)
  real(p_),intent(out):: efield(0:ngrids-1)
  !  real(p_)::L
  type(C_PTR) :: plan
  complex(C_DOUBLE_COMPLEX) :: in(0:ngrids-1), out(0:ngrids-1)
  complex(p_):: rho_hat(0:ngrids-1),efield_hat(0:ngrids-1),w
  complex(p_):: rho_hat2(0:ngrids-1)
  complex(p_),parameter:: ii=(0._p_,1._p_)
  integer:: i,j,k
  real(p_):: kvector(0:ngrids-1)

  !----calculate the DFT by naive method, tested,works
!!$  w=exp(ii*twopi/ngrids) !Eq. (12.2.1) in Numerical Recipe book.
!!$  rho_hat2=0.0
!!$  do i=0,ngrids-1 !calculate the DFT by using the naive method: direct calculation of the summation
!!$     do k=0,ngrids-1
!!$        rho_hat2(i)=rho_hat2(i)+w**(i*k)*rho(k) !Eq. (12.2.2) in Numerical Recipe book.
!!$     enddo
!!$  enddo
  !---------------

  in=rho
  plan = fftw_plan_dft_1d(ngrids, in,out,FFTW_BACKWARD,FFTW_ESTIMATE)
  call fftw_execute_dft(plan, in, out)
  call fftw_destroy_plan(plan)
  rho_hat=out

 
!call filter(rho_hat,ngrids)

!!$  in=rho_hat
!!$  plan = fftw_plan_dft_1d(ngrids, in,out, FFTW_FORWARD,FFTW_ESTIMATE)
!!$  call fftw_execute_dft(plan, in, out)
!!$  call fftw_destroy_plan(plan)
!!$
!!$  do j=0,ngrids-1 !test whether a forward FFT and then a backword fft will reproduces the original data. (results indicate the original data are reproduced)
!!$     write(*,*) j, real(out(j))/ngrids,real(rho(j)), imag(out(j))/ngrids, 0._p_
!!$  enddo

!!$do j=0,ngrids-1
!!$ write(*,*) j, real(rho_hat2(j)),real(rho_hat(j)), imag(rho_hat2(j)), imag(rho_hat(j))
!!$enddo

  !  call filter(rho_hat,ngrids) !artificially reducing/filtering the high-k component

  !  L=dx*(ngrids-1)
  !  phi_hat(0)=0._p_ !zero frequency can not be solved from the Fourier transformaed Poisson's equation, but we know that it should be zero.

!!$  do j=1,ngrids-1
!!$     if(j<ngrids/2) then
!!$        phi_hat(j)=-rho_hat(j)/(twopi*j/(ngrids*dx))**2
!!$     else
!!$        phi_hat(j)=-rho_hat(j)/(twopi*(j-ngrids)/(ngrids*dx))**2
!!$     endif
!!$  enddo

  do j=0,ngrids-1
     if(j<ngrids/2) then
        kvector(j)=twopi*j/(ngrids*dx)
     else
        kvector(j)=twopi*(j-ngrids)/(ngrids*dx)
     endif
  enddo

!!$phi_hat=-rho_hat/kvector**2
!!$efield_hat=-ii*kvector*phi_hat
  do j=1,ngrids-1
     efield_hat(j)=ii*rho_hat(j)/kvector(j)
  enddo
  efield_hat(0)=(0._p_,0._p_) !the zero frequency component can not be solved from the above equation in k space becuase k=0 and k appears in the denominator, but we know that it should be zero

!!$  do j=ngrids/2+1,ngrids-1
!!$          phi_hat(j)=real(phi_hat(ngrids-j))-ii*imag(phi_hat(ngrids-j)) !to make sure phi_hat(-f)=phi_hat_^star(f)
!!$  enddo
!call filter(efield_hat,ngrids)
!!$do j=0,ngrids-1
!!$     write(*,*) j,phi_hat(j)
!!$  enddo
!!$
  in=efield_hat
  plan = fftw_plan_dft_1d(ngrids, in,out, FFTW_FORWARD,FFTW_ESTIMATE)
  call fftw_execute_dft(plan, in, out)
  call fftw_destroy_plan(plan)

  do j=0,ngrids-1
     efield(j)=out(j)/ngrids !ngrids is the normalizing factor, refer to my notes on Fourier analysis
  enddo


!!$ w=exp(-ii*twopi/ngrids) !Eq. (12.2.1) in Numerical Recipe book.
!!$  phi=0.0
!!$  do i=0,ngrids-1 !calculate the DFT by using the naive method: direct calculation of the summation
!!$     do k=0,ngrids-1
!!$        phi(i)=phi(i)+w**(i*k)*phi_hat(k) !Eq. (12.2.2) in Numerical Recipe book.
!!$     enddo
!!$  enddo
!!$
!!$phi=phi/ngrids

end subroutine efield_solver_dft


subroutine filter(rho_hat,ngrids)
  !artificially reducing/removing the high-k component
  use precision,only:p_
  implicit none
  integer,intent(in):: ngrids
  complex(p_),intent(inout):: rho_hat(0:ngrids-1)
  integer:: j
  integer,parameter:: js=8 !keep only the js harmonic
  !  do j=ngrids/4,ngrids/2
     rho_hat(0)=0.0
    do j=1,js-1
     rho_hat(j)=0.0*rho_hat(j) !positive wavenumber part
     rho_hat(ngrids-j)=0.0*rho_hat(ngrids-j) !negative wavenumber part
  enddo
 do j=js+1,ngrids/2
     rho_hat(j)=0.0*rho_hat(j) !positive wavenumber part
     rho_hat(ngrids-j)=0.0*rho_hat(ngrids-j) !negative wavenumber part
  enddo
end subroutine filter

subroutine my_fft(ngrids,dx,rho,rho_hat) 
  !(use FFTW library)
  use, intrinsic :: iso_c_binding
  use  precision,only:p_
  use  constants,only:zero,one,twopi
  implicit none
  include 'fftw3.f03'
  integer,intent(in):: ngrids
  real(p_),intent(in):: dx,rho(0:ngrids-1)
  complex(p_),intent(out):: rho_hat(0:ngrids-1)
  type(C_PTR) :: plan
  complex(C_DOUBLE_COMPLEX) :: in(0:ngrids-1), out(0:ngrids-1)

  integer:: j

  do j=0,ngrids-1
     in(j)=rho(j)
  enddo
  plan = fftw_plan_dft_1d(ngrids, in,out,FFTW_BACKWARD,FFTW_ESTIMATE)
  call fftw_execute_dft(plan, in, out)
  call fftw_destroy_plan(plan)
  rho_hat=out

end subroutine my_fft




subroutine poisson_solver_fd(ngrids,dx,rho,phi) 
  !use the finite difference method to solve Poisson's equation to get the electric potential phi
  use  precision,only:p_
  use  constants,only:zero,one,two
  implicit none
  integer,intent(in):: ngrids
  real(p_),intent(in):: dx,rho(0:ngrids-1)
  real(p_),intent(out):: phi(0:ngrids-1)
  real(p_):: a(ngrids-2),b(ngrids-2),c(ngrids-2),r(ngrids-2),u(ngrids-2)
  integer:: i

  do i=1,ngrids-2
     a(i)=one
     b(i)=-two
     c(i)=one
     r(i)=-dx**2*rho(i)
  enddo

  call tridiagonal(a,b,c,r,u,ngrids-2)

  do i=1,ngrids-2
     phi(i)=u(i)
  enddo

  phi(0)=zero
  phi(ngrids-1)=zero

end subroutine poisson_solver_fd


SUBROUTINE tridiagonal(a,b,c,r,u,n)
  !This routine is to solve the tridiagonal matrix. This routine is from Numerical Recipes, 
  !with some minor modification by yjhu
  use precision,only:p_
  implicit none
  !  integer,PARAMETER:: NMAX=5000
  INTEGER,intent(in):: n
  REAL(p_),intent(in):: a(n),b(n),c(n),r(n)
  real(p_),intent(out)::u(n)
  INTEGER j
  ! REAL(p_) bet,gam(NMAX)
  REAL(p_) bet,gam(n)
  if(b(1).eq.0.) stop 'tridag: The tridiangle matrix equation need rewriting'
  bet=b(1)
  u(1)=r(1)/bet
  do  j=2,n
     gam(j)=c(j-1)/bet
     bet=b(j)-a(j)*gam(j)
     if(bet.eq.0.) stop 'tridag failed'
     u(j)=(r(j)-a(j)*u(j-1))/bet
  enddo
  do  j=n-1,1,-1
     u(j)=u(j)-gam(j+1)*u(j+1)
  enddo
END SUBROUTINE tridiagonal
